// ////////////////////////////////////////////////////
//
// StdView.cpp : implementation of the CStdView class
// Based on class COGLView : public CView
// Inspect COGLView for standard OpenGL initialization
//
////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Std.h"

#include "StdDoc.h"
#include "StdView.h"

#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////
// You can put definitions, variables and helperfunctions
// here. Functions that does not rely on CStdView member
// variables or functions

#define A_BOX		1
#define A_SPHERE	2
#define A_CYLINDER	3
#define A_POLYGON	4
#define A_DISK		5
#define A_BEZIER	6


//////////////////////////////////////////////////////////
// CStdView

IMPLEMENT_DYNCREATE(CStdView, COGLView)

BEGIN_MESSAGE_MAP(CStdView, COGLView)
	//{{AFX_MSG_MAP(CStdView)
	ON_WM_SIZE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_COMMAND(ID_VIEW_HOME, OnViewHome)
	ON_COMMAND(ID_SHAPE_BOX, OnShapeBox)
	ON_COMMAND(ID_SHAPE_CYLINDER, OnShapeCylinder)
	ON_COMMAND(ID_SHAPE_DISK, OnShapeDisk)
	ON_COMMAND(ID_SHAPE_POLYGON, OnShapePolygon)
	ON_COMMAND(ID_SHAPE_SPHERE, OnShapeSphere)
	ON_COMMAND(ID_BITMAP_LOAD, OnBitmapLoad)
	ON_UPDATE_COMMAND_UI(ID_SHAPE_BOX, OnUpdateShapeBox)
	ON_UPDATE_COMMAND_UI(ID_SHAPE_CYLINDER, OnUpdateShapeCylinder)
	ON_UPDATE_COMMAND_UI(ID_SHAPE_DISK, OnUpdateShapeDisk)
	ON_UPDATE_COMMAND_UI(ID_SHAPE_POLYGON, OnUpdateShapePolygon)
	ON_UPDATE_COMMAND_UI(ID_SHAPE_SPHERE, OnUpdateShapeSphere)
	ON_COMMAND(ID_VIEW_HIDE_BACK, OnViewHideBack)
	ON_UPDATE_COMMAND_UI(ID_VIEW_HIDE_BACK, OnUpdateViewHideBack)
	ON_COMMAND(ID_SHAPE_BEZIER, OnShapeBezier)
	ON_UPDATE_COMMAND_UI(ID_SHAPE_BEZIER, OnUpdateShapeBezier)
	ON_COMMAND(ID_VIEW_BASIC_MAT, OnViewBasicMat)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////
// CStdView construction/destruction

CStdView::CStdView()
{
	SetHomePos();
	//m_theShape=A_BEZIER;
	m_theShape=A_POLYGON;
	m_hideBack=FALSE;
	m_Material=WARM_WHITE;
	m_Vx=0.0f;
	m_Vy=0.0f;

	m_hRC=NULL;
};

CStdView::~CStdView()
{}

////////////////////////////////////////////////////////////
// CStdView diagnostics
// You should usually not touch these:
#ifdef _DEBUG
void CStdView::AssertValid() const
{COGLView::AssertValid();}
void CStdView::Dump(CDumpContext& dc) const
{COGLView::Dump(dc);}

CStdDoc* CStdView::GetDocument() // non-debug version is inline
{	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CStdDoc)));
	return (CStdDoc*)m_pDocument;
}
#endif //_DEBUG

//////////////////////////////////////////////////////////////
// CStdView operations

void CStdView::DrawScene()
{
	glEnable(GL_TEXTURE_2D);

	glShadeModel(GL_SMOOTH);

    // Clear the color and depth buffers.
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	if(m_hideBack)
	{
		// will not se backsides
		glCullFace(GL_BACK);
		glEnable(GL_CULL_FACE);
	}
	else
		glDisable(GL_CULL_FACE);

		
	// Set up for scene
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(	0.0f,0.0f,20.0f,
				0.0f,0.0f,0.0f,
				0.0f,1.0f,0.0f);

		// tilt it around x
	glRotatef(m_Vx,1.0f,0.0f,0.0f);
	// tilt it around y
	glRotatef(m_Vy,0.0f,1.0f,0.0f);

	CMaterial::SetMaterial(m_Material);
	glPushAttrib(GL_TEXTURE_BIT);

	switch (m_theShape)
	{
		case A_BOX:
			DrawTexBox(7.0f,7.0f,7.0f);
			break;

		case A_SPHERE:
			DrawSphere();
			break;

		case A_CYLINDER:
			DrawCylinder();
			break;

		case A_POLYGON:
			DrawPolygon();
			break;

		case A_DISK:
			DrawDisk();
			break;

		case A_BEZIER:
			DrawBezier();
			break;

		default:;

	}
	glPopAttrib();

	glFlush();
}

void CStdView::InitializeRContext()
{
	// Here we set the attributes that will be constant
	// during the rendering contexts lifetime
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LESS);
    
	// prepare ligthsource
	GLfloat ambient[] = {0.2f,0.2f,0.2f,1.0f };
    GLfloat diffuse[] = {1.0f,1.0f,1.0f,1.0f };
    GLfloat position[] = {4.0f,0.0f,6.0f,0.0f };
	GLfloat lmodel_ambient[] = {0.4f,0.4f,0.4f,1.0f };


    glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
    glLightfv(GL_LIGHT0, GL_POSITION, position);
	
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);

	// smooth the drawing
	glShadeModel(GL_SMOOTH);
	glEnable(GL_NORMALIZE);

    // set background to light gray
	glClearColor(0.8f, 0.8f, 0.8f, 0.0f);
	glClearColor(1.0f, 1.0f, 1.0f, 0.0f);

	// load images
	LoadTexture("tower.bmp");
}


void CStdView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	glViewport(0, 0, cx, cy);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	gluPerspective(40.0f, (GLdouble)cx/(GLdouble)cy, 0.1f, 60.0f);
	glMatrixMode(GL_MODELVIEW);
	Invalidate(NULL);
}

void CStdView::SetHomePos()
{
	// initialize angles, see DrawScene
	m_Vz=0.0f;
	m_Vx=0.0f;
	m_Vy=0.0f;

	// Viewpoint
	m_EyeZ=6.0f;
	m_EyeY=m_EyeX=0.0f;

	// moving, animation
	m_dvy=0.0f;
	m_dvx=0.0f;
	m_bHasStopped=FALSE;
}

BOOL CStdView::LoadTexture(CString name)
{
	m_theImage.Load(name,TRANS_DIY);
	BOOL res=((m_theImage.GetWidth()>0)&&(m_theImage.GetHeight()>0)
			&&(m_theImage.GetBitsPtr()!=NULL));
	
	if(res)
	{
		// and prepare texture parameters
		// I am uncertain of the effect of alignment
		GLint iAlign;
		glGetIntegerv(GL_UNPACK_ALIGNMENT,&iAlign);
		glPixelStorei(GL_UNPACK_ALIGNMENT,iAlign);
	}
	return res;

}

void CStdView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// User has pressed left button.Make a note
	// of this and prepare for intuitive rotation of scene
	m_PLast=point;
	SetCapture();
	CView::OnLButtonDown(nFlags, point);
}

void CStdView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// free capture and terminate rotation
	ReleaseCapture();
	CView::OnLButtonUp(nFlags, point);
	
	if(m_bHasStopped)
	{
		// lower flag that indicates that this 
		// is the end of a stop click
		m_bHasStopped=FALSE;
	}
	else if((fabs(m_dvy)>1.0f)||(fabs(m_dvx)>1.0f))
	{
		// rotate around the x- and y-axis 
		// with degreesteps from interaction
		// stop rotation on mousedown
		MSG msg;
		do
		{
			m_Vy+=m_dvy;
			m_Vx+=m_dvx;
			RedrawWindow();
		}while (!PeekMessage(
					&msg,
					GetSafeHwnd(),
					WM_LBUTTONDOWN,
					WM_LBUTTONDOWN,
					PM_REMOVE)	// to make safer stop
					);
		// raise flag that indicates that we have had
		// a stop-click
		m_bHasStopped=TRUE;
	}
}

void CStdView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// if left button is down and this is not a result of
	// a stop-click
	if((nFlags & MK_LBUTTON)&&(!m_bHasStopped))
	{	
		// and has moved a significant dist
		if((abs(m_PLast.x-point.x)>2)||(abs(m_PLast.y-point.y)>2))
		{
			// then we find an intuitive rot angle
			CRect R;
			GetClientRect(&R);

			m_dvy=90.0f*((GLfloat (point.x-m_PLast.x))/GLfloat (R.Width()));
			m_dvx=90.0f*((GLfloat (point.y-m_PLast.y))/GLfloat (R.Height()));
			
			m_Vy+=m_dvy;
			m_Vx+=m_dvx;
			
			// rememeber point and force immediate redraw 
			m_PLast=point;
			RedrawWindow();
		}
	}
}

void CStdView::OnViewHome() 
{
	SetHomePos();
	RedrawWindow();
}

void CStdView::OnViewBasicMat() 
{
	CMaterial m;
	if(m.DoModal()==IDOK)
	{
		m_Material=m.Get_Current_Index();
		RedrawWindow();
	}
	
}


void CStdView::OnShapeBox() 
{
	m_theShape=A_BOX;
	RedrawWindow();
}

void CStdView::OnShapeCylinder() 
{
	m_theShape=A_CYLINDER;
	RedrawWindow();
}

void CStdView::OnShapeDisk() 
{
	m_theShape=A_DISK;
	RedrawWindow();
}

void CStdView::OnShapePolygon() 
{
	m_theShape=A_POLYGON;
	RedrawWindow();
}

void CStdView::OnShapeSphere() 
{
	m_theShape=A_SPHERE;
	RedrawWindow();
}

void CStdView::OnShapeBezier() 
{
	m_theShape=A_BEZIER;
	RedrawWindow();
}


void CStdView::OnUpdateShapeBezier(CCmdUI* pCmdUI) 
{	pCmdUI->SetCheck(m_theShape==A_BEZIER);}

void CStdView::OnUpdateShapeBox(CCmdUI* pCmdUI) 
{	pCmdUI->SetCheck(m_theShape==A_BOX);}

void CStdView::OnUpdateShapeCylinder(CCmdUI* pCmdUI) 
{	pCmdUI->SetCheck(m_theShape==A_CYLINDER);}

void CStdView::OnUpdateShapeDisk(CCmdUI* pCmdUI) 
{	pCmdUI->SetCheck(m_theShape==A_DISK);}

void CStdView::OnUpdateShapePolygon(CCmdUI* pCmdUI) 
{	pCmdUI->SetCheck(m_theShape==A_POLYGON);}

void CStdView::OnUpdateShapeSphere(CCmdUI* pCmdUI) 
{	pCmdUI->SetCheck(m_theShape==A_SPHERE);}


void CStdView::OnBitmapLoad() 
{
	// load a bitmap to display
	CFileDialog fd(TRUE,".bmp","default.bmp");
	if(fd.DoModal()==IDOK)
	{
		CString fn=fd.GetPathName();
		if(LoadTexture(fn))
			RedrawWindow();
		else
			AfxMessageBox("load error");
	}
}

void CStdView::OnViewHideBack() 
{
	m_hideBack=!m_hideBack;
	RedrawWindow();
}

void CStdView::OnUpdateViewHideBack(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_hideBack);
	
}


void CStdView::DrawBezier()
{
	/////////////////////////////////////
	// define ctrlpoints for beziersuface
	GLfloat ctrlpoints[4][4][3] = // [v][u][xyz] 
	{
	{{0.0f,0.0f,4.0f},{3.0f,0.0f,2.0f},{9.0f,3.0f,-1.0f},{9.0f,0.0f,2.0f}},
	{{0.0f,3.0f,1.0f},{3.0f,3.0f,3.0f},{9.0f,3.0f,0.0f},{12.0f,3.0f,-1.0f}}, 
	{{0.0f,9.0f,4.0f},{3.0f,9.0f,0.0f},{9.0f,9.0f,3.0f}, {12.0f,9.0f,4.0f}}, 
	{{0.0f,12.0f,-2.0f},{3.0f,12.0f,-2.0f},{9.0f,12.0f,0.0f},{12.0f,12.0f,-1.0f}}
	};
	
	/*
	// make it swing a little
	for(int x=1;x<3;x++)
	for(int y=1;y<3;y++)
	{
		ctrlpoints[x][y][2]=(GLfloat)(rand()%6-3);
	}
	*/

	glPushMatrix();
	glTranslatef(-6.0f,-6.0f,0.0f);



	glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_DECAL);


	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);//GL_CLAMP
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);//GL_CLAMP
	
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_MAP2_VERTEX_3);
	glEnable(GL_MAP2_TEXTURE_COORD_1);
	glEnable(GL_MAP2_TEXTURE_COORD_2);


	glTexImage2D(GL_TEXTURE_2D,0,3,
				m_theImage.GetWidth(),m_theImage.GetHeight(),
				0,GL_RGB,GL_UNSIGNED_BYTE,
				m_theImage.GetBitsPtr());


	// map geometry
	glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,4,0.0f,1.0f,12,4,
			&ctrlpoints[0][0][0]);

	// map texture 
	GLfloat txpts[2][2][2] =	{{{0.0, 0.0}, {1.0, 0.0}}, 
								{{0.0, 1.0}, {1.0, 1.0}}};
	glMap2f(GL_MAP2_TEXTURE_COORD_2, 0, 1, 2, 2, 
			0, 1, 4, 2, &txpts[0][0][0]);

	glMapGrid2f(20,0.0f,1.0f,20,0.0f, 1.0f);

	glEvalMesh2(GL_FILL, 0, 20, 0, 20);
	glPopMatrix();

}

void CStdView::DrawPolygon()
{
	// check out GL_CLAMP or GL_REPEAT, combined with 1x1 or nxn
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);// GL_CLAMP | GL_REPEAT
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);

	
	// check out GL_NEAREST and GL_LINEAR
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	
	glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);//GL_DECAL | GL_REPLACE | GL_MODULATE | GL_BLEND
	
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);

	glTexImage2D(GL_TEXTURE_2D,0,3,
				m_theImage.GetWidth(),
				m_theImage.GetHeight(),
				0,
				GL_RGB,
				GL_UNSIGNED_BYTE,
				m_theImage.GetBitsPtr());


	//  and draw a square with nxn images, se clamp and repeat
	GLfloat n=1.0;
	glBegin(GL_POLYGON);
		glTexCoord2f(0.0f,0.0f); glVertex3f(-6.5f,-6.5f,0.1f);
		glTexCoord2f(n,0.0f); glVertex3f(6.5f,-6.5f,0.1f);
		glTexCoord2f(n,n); glVertex3f(6.5f,6.5f,0.1f);
		glTexCoord2f(0.0f,n); glVertex3f(-6.5f,6.5f,0.1f);
	glEnd();
	/*
	// draw a single drawing 1x1
	glBegin(GL_POLYGON);
		glTexCoord2f(0.0f,0.0f); glVertex3f(-6.5f,-6.5f,0.1f);
		glTexCoord2f(1.0f,0.0f); glVertex3f(6.5f,-6.5f,0.1f);
		glTexCoord2f(1.0f,1.0f); glVertex3f(6.5f,6.5f,0.1f);
		glTexCoord2f(0.0f,1.0f); glVertex3f(-6.5f,6.5f,0.1f);
	glEnd();
*/

}

void CStdView::DrawDisk()
{
	GLUquadricObj *glpQ;
	glpQ = gluNewQuadric();
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
	//glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_DECAL);//GL_MODULATE
	glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);

	glTexImage2D(GL_TEXTURE_2D,0,3,
				m_theImage.GetWidth(),
				m_theImage.GetHeight(),
				0,
				GL_RGB,
				GL_UNSIGNED_BYTE,
				m_theImage.GetBitsPtr());


	gluQuadricTexture(glpQ,TRUE);
	gluDisk(glpQ,0.0f,3.0f,20,20);
	gluDeleteQuadric(glpQ);
}

void CStdView::DrawCylinder()
{
	GLUquadricObj *glpQ;
	glpQ = gluNewQuadric();
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
	glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_DECAL);//GL_MODULATE

	glTexImage2D(GL_TEXTURE_2D,0,3,
				m_theImage.GetWidth(),
				m_theImage.GetHeight(),
				0,
				GL_RGB,
				GL_UNSIGNED_BYTE,
				m_theImage.GetBitsPtr());
	gluQuadricTexture(glpQ,TRUE);
	gluCylinder(glpQ,3.0f,3.0f,7.0f,20,20);

	gluDeleteQuadric(glpQ);
}

void CStdView::DrawSphere()
{
	GLUquadricObj *glpQ;
	glpQ = gluNewQuadric();
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);//GL_CLAMP
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);//GL_CLAMP
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
	glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_DECAL);//GL_MODULATE

	glTexImage2D(GL_TEXTURE_2D,0,3,
				m_theImage.GetWidth(),
				m_theImage.GetHeight(),
				0,
				GL_RGB,
				GL_UNSIGNED_BYTE,
				m_theImage.GetBitsPtr());


	gluQuadricTexture(glpQ,TRUE);
	gluSphere(glpQ,5.0f,20,20);

	gluDeleteQuadric(glpQ);
}

void CStdView::DrawTexBox(GLfloat dx,GLfloat dy,GLfloat dz)
{
	// draws the sides of a cube 
	// (-dx/2,-dx/2,-dz/2)-(dx/2,dx/2,dz/2)
	GLfloat dX=dx/2.0f;
	GLfloat dY=dy/2.0f;
	GLfloat dZ=dz/2.0f;

	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
	//glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_DECAL);
	glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);


	glTexImage2D(GL_TEXTURE_2D,0,3,
				m_theImage.GetWidth(),
				m_theImage.GetHeight(),
				0,
				GL_RGB,
				GL_UNSIGNED_BYTE,
				m_theImage.GetBitsPtr());

	glBegin(GL_POLYGON);//f1: front
		glNormal3f(0.0f,-1.0f,0.0f);
		glTexCoord2f(0.0f,0.0f); glVertex3f(-dX,-dY,-dZ);
		glTexCoord2f(1.0f,0.0f); glVertex3f(dX,-dY,-dZ);
		glTexCoord2f(1.0f,1.0f); glVertex3f(dX,-dY,dZ);
		glTexCoord2f(0.0f,1.0f); glVertex3f(-dX,-dY,dZ);
	glEnd();
	glBegin(GL_POLYGON);//f2: bottom
		glNormal3f(0.0f,0.0f,-1.0f);
		glTexCoord2f(0.0f,0.0f); glVertex3f(-dX,-dY,-dZ);
		glTexCoord2f(2.0f,0.0f); glVertex3f(-dX,dY,-dZ);
		glTexCoord2f(2.0f,2.0f); glVertex3f(dX,dY,-dZ);
		glTexCoord2f(0.0f,2.0f); glVertex3f(dX,-dY,-dZ);
	glEnd();
	glBegin(GL_POLYGON);// f3:back
		glNormal3f(0.0f,1.0f,0.0f);
		glTexCoord2f(0.0f,0.0f); glVertex3f(-dX,dY,-dZ);
		glTexCoord2f(3.0f,0.0f); glVertex3f(-dX,dY,dZ);
		glTexCoord2f(3.0f,3.0f); glVertex3f(dX,dY,dZ);
		glTexCoord2f(0.0f,3.0f); glVertex3f(dX,dY,-dZ);
	glEnd();
	glBegin(GL_POLYGON);// f4: top
		glNormal3f(0.0f,0.0f,1.0f);
		glTexCoord2f(0.0f,0.0f); glVertex3f(-dX,-dY,dZ);
		glTexCoord2f(2.0f,0.0f); glVertex3f(dX,-dY,dZ);
		glTexCoord2f(2.0f,3.0f); glVertex3f(dX,dY,dZ);
		glTexCoord2f(0.0f,3.0f); glVertex3f(-dX,dY,dZ);
	glEnd();
	glBegin(GL_POLYGON);//f5: left
		glNormal3f(-1.0f,0.0f,0.0f);
		glTexCoord2f(0.0f,0.0f); glVertex3f(-dX,dY,-dZ);
		glTexCoord2f(3.0f,0.0f); glVertex3f(-dX,-dY,-dZ);
		glTexCoord2f(3.0f,2.0f); glVertex3f(-dX,-dY,dZ);
		glTexCoord2f(0.0f,2.0f); glVertex3f(-dX,dY,dZ);
	glEnd();
	glBegin(GL_POLYGON);//f6: right
		glNormal3f(1.0f,0.0f,0.0f);
		glTexCoord2f(0.0f,0.0f); glVertex3f(dX,-dY,-dZ);
		glTexCoord2f(4.0f,0.0f); glVertex3f(dX,dY,-dZ);
		glTexCoord2f(4.0f,4.0f); glVertex3f(dX,dY,dZ);
		glTexCoord2f(0.0f,4.0f); glVertex3f(dX,-dY,dZ);
	glEnd();
}

